using Avalonia;
using Avalonia.Controls;
using Avalonia.Controls.ApplicationLifetimes;
using Avalonia.Markup.Xaml;
using Avalonia.Styling;
using Avalonia.Themes.Fluent;
using Avalonia.Themes.Simple;

using AvaloniaCoreRTDemo.Interfaces;
using AvaloniaCoreRTDemo.Windows;

namespace AvaloniaCoreRTDemo;

public sealed class App : Application, IThemeSwitch
{
    private FluentTheme _fluentTheme = default!;
    private SimpleTheme _simpleTheme = default!;
    private IStyle _fluentDataGrid = default!;
    private IStyle _simpleDataGrid = default!;

    private ApplicationTheme _currentTheme;

    public override void Initialize()
    {
        AvaloniaXamlLoader.Load(this);
        Name = "AvaloniaCoreRTDemo";
    }

    public override void OnFrameworkInitializationCompleted()
    {
        InitializeThemes();
        if (ApplicationLifetime is IClassicDesktopStyleApplicationLifetime desktop)
        {
            desktop.MainWindow = new MainWindow();
            DataContext = desktop.MainWindow.DataContext;
        }
        base.OnFrameworkInitializationCompleted();
    }

    private void InitializeThemes()
    {
        _simpleTheme = new SimpleTheme();
        _fluentTheme = new FluentTheme();

        _fluentDataGrid = (IStyle)Resources["fluentDataGrid"]!;
        _simpleDataGrid = (IStyle)Resources["simpleDataGrid"]!;

        Styles.Add(_fluentTheme);
        Styles.Add(_fluentDataGrid);
        _currentTheme = ApplicationTheme.FluentLight;
    }

    ApplicationTheme IThemeSwitch.Current => _currentTheme;

    void IThemeSwitch.ChangeTheme(ApplicationTheme theme)
    {
        if (theme == _currentTheme)
            return;

        bool themeChanged = theme switch
        {
            ApplicationTheme.SimpleLight
                => _currentTheme is ApplicationTheme.FluentDark or ApplicationTheme.FluentLight,
            ApplicationTheme.SimpleDark
                => _currentTheme is ApplicationTheme.FluentDark or ApplicationTheme.FluentLight,
            ApplicationTheme.FluentLight
                => _currentTheme is ApplicationTheme.SimpleLight or ApplicationTheme.SimpleDark,
            ApplicationTheme.FluentDark
                => _currentTheme is ApplicationTheme.SimpleLight or ApplicationTheme.SimpleDark,
            _ => throw new ArgumentOutOfRangeException(nameof(theme), theme, null)
        };

        _currentTheme = theme;
        switch (theme)
        {
            case ApplicationTheme.SimpleLight:
                SetValue(ThemeVariantScope.ActualThemeVariantProperty, ThemeVariant.Light);
                Styles[0] = _simpleTheme;
                Styles[1] = _simpleDataGrid;
                break;
            case ApplicationTheme.SimpleDark:
                SetValue(ThemeVariantScope.ActualThemeVariantProperty, ThemeVariant.Dark);
                Styles[0] = _simpleTheme;
                Styles[1] = _simpleDataGrid;
                break;
            case ApplicationTheme.FluentLight:
                SetValue(ThemeVariantScope.ActualThemeVariantProperty, ThemeVariant.Light);
                Styles[0] = _fluentTheme;
                Styles[1] = _fluentDataGrid;
                break;
            case ApplicationTheme.FluentDark:
                SetValue(ThemeVariantScope.ActualThemeVariantProperty, ThemeVariant.Dark);
                Styles[0] = _fluentTheme;
                Styles[1] = _fluentDataGrid;
                break;
        }

        if (themeChanged && ApplicationLifetime is IClassicDesktopStyleApplicationLifetime desktop)
        {
            MainWindow oldWindow = (desktop.MainWindow as MainWindow)!;
            MainWindow newWindow = new MainWindow(oldWindow);

            desktop.MainWindow = newWindow;
            DataContext = newWindow.DataContext;

            oldWindow.Hide();
            newWindow.Show();
            oldWindow?.Close();
        }
    }
}
